@@ -1,11 +1,12 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/uproot4/blob/main/LICENSE
+# BSD 3-Clause License; see https://github.com/scikit-hep/uproot5/blob/main/LICENSE
 
 """
 This module defines a versionless model of ``TString``.
 """
-
+from __future__ import annotations
 
 import uproot
+import uproot._awkwardforth
 
 
 class Model_TString(uproot.model.Model, str):
@@ -21,11 +22,40 @@ class Model_TString(uproot.model.Model, str):
     def read_members(self, chunk, cursor, context, file):
         if self.is_memberwise:
             raise NotImplementedError(
-                """memberwise serialization of {}
-in file {}""".format(
-                    type(self).__name__, self.file.file_path
-                )
+                f"""memberwise serialization of {type(self).__name__}
+in file {self.file.file_path}"""
             )
+        forth_obj = uproot._awkwardforth.get_forth_obj(context)
+        if forth_obj is not None:
+            offsets_num = uproot._awkwardforth.get_first_key_number(context)
+            data_num = offsets_num + 1
+            nested_forth_stash = uproot._awkwardforth.Node(
+                f"node{offsets_num} TString",
+                form_details={
+                    "class": "ListOffsetArray",
+                    "offsets": "i64",
+                    "content": {
+                        "class": "NumpyArray",
+                        "primitive": "uint8",
+                        "inner_shape": [],
+                        "parameters": {"__array__": "char"},
+                        "form_key": f"node{data_num}",
+                    },
+                    "parameters": {"__array__": "string"},
+                    "form_key": f"node{offsets_num}",
+                },
+            )
+            nested_forth_stash.pre_code.append(
+                f" stream !B-> stack dup 255 = if drop stream !I-> stack then dup node{offsets_num}-offsets +<- stack stream #!B-> node{data_num}-data\n"
+            )
+            nested_forth_stash.header_code.append(
+                f"output node{offsets_num}-offsets int64\noutput node{data_num}-data uint8\n"
+            )
+            nested_forth_stash.init_code.append(
+                f"0 node{offsets_num}-offsets <- stack\n"
+            )
+            forth_obj.add_node(nested_forth_stash)
+
         self._data = cursor.string(chunk, context)
 
     def postprocess(self, chunk, cursor, context, file):
@@ -44,9 +74,7 @@ in file {}""".format(
             version = ""
         else:
             version = f" (version {self.class_version})"
-        return "<{}{} {} at 0x{:012x}>".format(
-            self.classname, version, str.__repr__(self), id(self)
-        )
+        return f"<{self.classname}{version} {str.__repr__(self)} at 0x{id(self):012x}>"
 
     def tojson(self):
         return str(self)
